/**
 * Darstellung von Ersatzteilen.
 *
 * <p>Ein Ersatzteil kann unter Angabe einer Teilenummer und einer optionalen Bezeichnung erstellt werden. Beispiel:</p>
 *
 * <table class="goikTableDefaults">
 *   <tr>
 *       <th>Code</th>
 *       <th>Ausgabe</th>
 *   </tr>
 *   <tr>
 *       <td>
 *           <pre><code class="java"> final Ersatzteil kurbelwelle = new Ersatzteil(12345, "Kurbelwelle");
 * System.out.println(kurbelwelle);
 *
 * final Ersatzteil brandneu =     // Bezeichnung noch
 *    new Ersatzteil(54321, null); // unbekannt.
 * System.out.println(brandneu); </code></pre>
 *       </td>
 *       <td>
 *           <pre> Kurbelwelle, Teilenummer:12345
 * Bezeichnung unbekannt, Teilenummer:54321 </pre>
 *       </td>
 *   </tr>
 * </table>
 *
 * <p>Zwei Instanzen der Klasse {@link de.hdm_stuttgart.mi.sd1.aufgabe2.Ersatzteil} sind bezüglich
 * <code>equals(...)</code> genau dann gleich sein, wenn ihre Teilenummern gleich sind. Die Bezeichnung der Teile ist
 * dabei irrelevant:</p>
 *
 * <table class="goikTableDefaults">
 *   <tr>
 *       <th>Code</th>
 *       <th>Ausgabe</th>
 *   </tr>
 *   <tr>
 *       <td>
 *           <pre><code class="java"> final Ersatzteil
 *       kurbelwelle = new Ersatzteil(12345, "Kurbelwelle"),
 *    schwurbelwelle = new Ersatzteil(12345, "Schwurbelwelle"), // Tippfehler?
 *           lenkrad = new Ersatzteil(54321, "Lenkrad");
 *
 * System.out.println("kurbelwelle gleich schwurbelwelle? " +
 *       kurbelwelle.equals(schwurbelwelle));
 *
 * System.out.println("kurbelwelle gleich lenkrad? " +
 *       kurbelwelle.equals(lenkrad)); </code></pre>
 *       </td>
 *       <td>
 *           <pre> kurbelwelle gleich schwurbelwelle? true
 * kurbelwelle gleich lenkrad? false </pre>
 *       </td>
 *   </tr>
 * </table>
 *
 * <p>Die <code>equals(...)</code> Methode kann in abgeleiteten Klassen nicht redefiniert werden.</p>
 *
 * <p>Die Klasse <code>Reifen</code> ist eine Spezialisierung der Klasse Ersatzteil. Diese hat gegenüber
 * {@link de.hdm_stuttgart.mi.sd1.aufgabe2.Ersatzteil} die zusätzliche Eigenschaft einer Maximalgeschwindigkeit:</p>
 *
 * <table class="goikTableDefaults">
 *   <tr>
 *       <th>Code</th>
 *       <th>Ausgabe</th>
 *   </tr>
 *   <tr>
 *       <td>
 *           <pre><code class="java"> final Reifen reifen =      // Reifen mit Teilenummer 1234 und
 *    new Reifen(1234, 160);  // Maximalgeschwindigkeit 160 Km pro Stunde. Die
 *                            // feste Teilebezeichnung "Reifen" wird in der
 *                            // Basisklasse {@link de.hdm_stuttgart.mi.sd1.aufgabe2.Ersatzteil} automatisch gesetzt.
 *
 * System.out.println(reifen); </code></pre>
 *       </td>
 *       <td>
 *           <pre> Reifen, Teilenummer:1234, Maximalgeschwindigkeit:160 Kmh </pre>
 *       </td>
 *   </tr>
 * </table>
 *
 */
package de.hdm_stuttgart.mi.sd1.aufgabe2;