package de.hdm_stuttgart.mi.sd1.aufgabe2;

import java.text.DecimalFormat;

/**
 * Darstellungen von Breitenangaben entweder als (Grad, Minuten, Sekunden) oder dezimal.
 *
 * <p>Längen- oder Breitenangaben können als (Grad, Winkelminuten, Winkelsekunden) dargestellt werden. Dabei gilt:</p>
 *
 * <table class="goikTableDefaults">
 *     <tr>
 *         <td style="width: 35em">
 *           <ul>
 *             <li>360° Grad entsprechen einem Vollkreis.</li>
 *             <li>60 Winkelminuten ergeben 1 Grad: 60´ == 1°.</li>
 *             <li>60 Winkelsekunden ergeben eine Winkelminute: 60´´ = 1´.</li>
 *           </ul>
 *         </td>
 *         <td style="width: 30em"><b>Beispiel:</b> Die geographische Breitenangabe von Stuttgart lautet
 *         (<b style="color: red;">48°</b>,<b style="color: green;">46′</b>,<b style="color: blue;">56″</b>). Die Angabe
 *         »Nord« wird in dieser Aufgabe ignoriert.</td>
 *     </tr>
 * </table>
 *
 * <p></p>
 *
 * <p>Alternativ gibt es auch eine <em>dezimale</em> Graddarstellung: Diese geht ebenfalls von einem Vollkreis zu 360°
 * aus. Allerdings werden die Minuten- und Sekundenanteile als Dezimalbruch angegeben. Die geographische Breite
 * (<b style="color: red;">48°</b>, <b style="color: green;">46′</b>, <b style="color: blue;">56″</b>) beispielsweise
 * von Stuttgart ergibt sich in dieser Darstellung zu:</p>
 *
 * \[\color{red}{48} + {\color{green}{46}\over 60} + {\color{blue}{56} \over {60 \times 60}} = 48.782222\dots \]
 *
 * <p>Instanzen von {@link Geo} können aus beiden Darstellungen erzeugt werden:</p>
 *
 * <pre>final Geo stuttgartBreite1 = new Geo(<b style="color: red;">48</b>, <b style="color: green;">46</b>,<b style="color: blue;">56</b>), // Grad, Minuten, Sekunden
 *          stuttgartBreite2 = new Geo(48.782222); // Dezimalangabe: Identische Breite bis auf Rundungsfehler</pre>
 *
 * <p>Jede Instanz von {@link Geo} stellt beide Darstellungen bereit:</p>
 *
 * <pre><code class="java"> final Geo breite = ... ;
 *
 * final int grad = breite.grad,
 *        minuten = breite.minuten,
 *       sekunden = breite.sekunden;
 *
 * final double breiteDezimal = breite.getDezimal();</code></pre>
 *
 * <p>Instanzen von {@link Geo} werden im Standard wie folgt ausgegeben:</p>
 *
 * <table class="goikTableDefaults">
 *     <tr>
 *         <th>Code</th>
 *         <th>Ausgabe</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo stuttgartBreite = new Geo(48.782222);
 * System.out.println(stuttgartBreite);</code></pre>
 *         </td>
 *         <td>(48°,46´,56´´)</td>
 *     </tr>
 * </table>
 *
 * <p>Das Format der Ausgabe kann zur Laufzeit global für alle Instanzen gewechselt werden. Dabei werden für die
 * Dezimaldarstellung genau sechs Nachkommastellen ausgegeben:</p>
 *
 * <table class="goikTableDefaults">
 *     <tr>
 *         <th>Code</th>
 *         <th>Ausgabe</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo stuttgartBreite = new Geo(48.782222);
 * Geo.nutzeDezimalDarstellung(true);
 * System.out.println(stuttgartBreite);</code></pre>
 *         </td>
 *         <td>48.782222</td>
 *     </tr>
 * </table>
 *
 * <p>Instanzen von Geo sollen auf Wertgleichheit geprüft werden können:</p>
 *
 * <table class="goikTableDefaults">
 *     <tr>
 *         <th>Code</th>
 *         <th>Ausgabe</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo
 *   g1 =      new Geo(33, 53, 19),
 *   g2 =      new Geo(45, 22, 13),
 *   g1_klon = new Geo(33, 53, 19);
 *
 * System.out.println(g1.equals(g2));
 * System.out.println(g1.equals(g1_klon));</code></pre>
 *         </td>
 *         <td>
 *             <pre> false
 * true</pre>
 *         </td>
 *     </tr>
 * </table>
 *
 * <p>Winkelangaben sollen aufsteigend sortierbar sein:</p>
 *
 * <table class="goikTableDefaults">
 *     <tr>
 *         <th>Code</th>
 *         <th>Ausgabe</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo
 * stuttgartBreite = new Geo(48, 46, 56),
 * beirutBreite = new Geo(33, 53, 19),
 * atlantaBreite = new Geo(33, 44, 56);
 *
 * final Geo[] breitenAngaben =
 *    {stuttgartBreite, beirutBreite, atlantaBreite};
 *
 * Arrays.sort(breitenAngaben);
 *
 * System.out.println("Breitengrade Stuttgart, Beirut und Atlanta"
 *     + "\naufsteigend sortiert:");
 *
 * for (final Geo g: breitenAngaben) {
 *   System.out.println(g);
 * }</code></pre>
 *         </td>
 *         <td>
 *             <pre> Breitengrade Stuttgart, Beirut und Atlanta
 * aufsteigend sortiert:
 * (33°,44´,56´´)
 * (33°,53´,19´´)
 * (48°,46´,56´´)</pre>
 *         </td>
 *     </tr>
 * </table>
 *
 * <p>Instanzen von {@link Geo} sind unveränderbar (immutable). Der folgende Code führt beispielsweise zu einem Compile
 * time Fehler:</p>
 *
 * <table class="goikTableDefaults">
 *     <tr>
 *         <th>Code</th>
 *         <th>Fehlermeldung Compiler</th>
 *     </tr>
 *     <tr>
 *         <td>
 *             <pre><code class="java"> final Geo g = new Geo(22);
 * g.sekunden = 44;</code></pre>
 *         </td>
 *         <td style="color: red;">Cannot assign a value to final variable 'sekunden'</td>
 *     </tr>
 * </table>
 *
 */
public class Geo implements Comparable<Geo> {

  private static DecimalFormat df = new DecimalFormat();
  static {
    df.setMaximumFractionDigits(6);
    df.setMinimumFractionDigits(6);
  }

  static private boolean nutzeDezimalDarstellung = false;

  /**
   * Die {@link #toString()} erzeugt im Standard einen String der Form »48° 23' 11''«. Alternativ
   * kann auch die dezimale Graddarstellung der Art »48.78222222...« als Default festgelegt werden.
   * @param nutzeDezimalDarstellung Für true erzeugt {@link #toString()} die Darstellung  »48.78222222...«,
   *                          ansonsten »48°,23',11''«.
   */
  public static void nutzeDezimalDarstellung(final boolean nutzeDezimalDarstellung) {
    Geo.nutzeDezimalDarstellung = nutzeDezimalDarstellung;
  }

  /**
   * Winkelangabe in Grad: Ein Vollkreis entspricht 360°.
   */
  public final int grad;
  /**
   * Winkelminuten: 60′ (sprich 60 Winkelminuten) entsprechen 1°.
   */
  public final int minuten;
  /**
   * Winkelsekunden: 60″ (sprich 60 Winkelsekunden) entsprechen 1′.
   */
  public final int sekunden;


  /**
   * Erzeuge Längen- oder Breitengradangabe.
   *
   * @param grad Winkelanteil in Grad
   * @param minuten Winkelanteil in Minuten
   * @param sekunden Winkelanteil in Sekunden
   */
  public Geo(final int grad, final int minuten, final int sekunden) {
    this.grad = grad;
    this.minuten = minuten;
    this.sekunden = sekunden;
  }

  /**
   * Längen- oder Breitenangabe aus Dezimaldarstellung.
   *
   * @param gradDezimal Dezimale Gradangabe, z.B. 43.221234
   */
  public Geo(final double gradDezimal) {

    final double grad = Math.floor(gradDezimal);

    final double minutenSekunden = 60 * (gradDezimal - grad);

    final double minutes = Math.floor(minutenSekunden);

    final double sekunden = 60 * (minutenSekunden - minutes);

    this.grad = (int) grad;
    this.minuten = (int)minutes;
    this.sekunden = (int) Math.round(sekunden);
  }

  public double getDezimal() {
    return grad + (minuten + sekunden / 60.) / 60;
  }

  @Override
  public String toString() {
    if (nutzeDezimalDarstellung) {
      return df.format(getDezimal());
    } else {
      return "(" + grad + "°," + minuten + "´," + sekunden + "´´)";
    }
  }

  @Override
  public int compareTo(final Geo o) {

    if (grad == o.grad) {
      if (minuten == o.minuten) {
        return sekunden - o.sekunden;
      } else {
        return minuten - o.minuten;
      }
    } else {
      return grad - o.grad;
    }
  }

  @Override
  public boolean equals(final Object other) {
    if (other instanceof Geo) {
      final Geo g = (Geo) other;

      return grad == g.grad && minuten == g.minuten && sekunden == g.sekunden;
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return 3600 * grad + 60 * minuten + sekunden;
  }
}
