package de.hdm_stuttgart.mi.sd1.task1;

/**
 * <p>Checking for Armstrong numbers.</p>
 */
public class _4_Armstrong {

    /**
     * <p>Check arbitrary integer values for being Armstrong numbers.</p>
     *
     * <p>An Armstrong number is every integer being equal to the sum of its own digits each raised to the
     * power of the number digits.</p>
     *
     * <p>Examples:
     * <ul>
     *     <li>153 having 3 digits is an Armstrong number since \(153 = 1^3 + 5^3 + 3^3 \).</li>
     *
     *     <li>Likewise 1634 having 4 digits is an Armstrong number as well: \(1634 = 1^4 + 6^4 + 3^4 + 4^4\).
     *     </li>
     *
     *     <li>On contrary »most« integer values i.e. 23 are no Armstrong numbers since e.g. \(23 \ne 2^2 + 3^2 \).
     *     </li>
     * </ul>
     *
     * </p>
     *
     * @param armstrongCandidate This value is to be examined whether or not it is an Armstrong number.
     * @return <code>true</code> if the value is an Armstrong number, <code>false</code> otherwise.
     *
     * <section class="implementationHints">
     *    <h3 class="implementationHints">Implementation hints:</h3>
     *    <ol>
     *        <li>
     *            <p>Turn a given integer into a string e.g. 153 --> <code>"153"</code>. The
     *            {@link Integer} class offers a suitable method.</p>
     *        </li>
     *
     *        <li>
     *            <p>Turn the string into an array of <code>char</code> e.g. "153" --> {'1', '5', '3'}. The
     *            {@link String} class offers a suitable method.</p>
     *        </li>
     *
     *        <li>
     *            <p>For each <code>char</code> access its »digit value« by subtracting <code>'0'</code>: For example
     *            the expression <code>'5' - '0'</code> amounts to 5 of type <code>int</code>.</p>
     *        </li>
     *    </ol>
     * </section>
     *
     */
    public static boolean isArmstrongNumber(final int armstrongCandidate) {
        final byte[] digits = int2digits(armstrongCandidate);
        return armstrongCandidate == digitPowerSum(digits);
    }

    /**
     * <p>De-composing an integer value into an array of its digits.</p>
     *
     * <p>Example: <code>int i = 12345</code> will be converted to {1, 2, 3, 4, 5}.</p>
     *
     * @param value The value to be converted.
     * @return The resulting array.
     *
     */
    static private byte[] int2digits(final int value) {
        final char[] digitCharacters = Integer.toString(value).toCharArray();
        final int digitCount = digitCharacters.length;

        final byte[] digits = new byte[digitCount];
        for (int i = 0; i < digitCount; i++) {
            digits[i] = (byte) (digitCharacters[i] - '0'); // Safe for {'0', '1', ..., '9'}
        }
        return digits;
    }

    /**
     * <p>Calculating an integer array's individual sum of components raised to the power of
     * the array's length.</p>
     *
     * <p>Example: {1, 5, 3} of length \(3\) will be turned into \( 1^3 + 5^3 + 3^3 \).</p>
     *
     * @param digits A non-null value.
     * @return The array's digit power sum.
     */
    static private int digitPowerSum(final byte[] digits) {

        int digitPowerSum = 0;
        for (final byte digit : digits) {
            int digitPower = digit;
            for (int i = 1; i < digits.length; i++) {
                digitPower *= digit;
            }
            digitPowerSum += digitPower;
        }
        return digitPowerSum;
    }

    private _4_Armstrong(){/* Ignore me: Suppressing default constructor javadoc generation:
                                    javadoc does not include private constructors by default */}
}
